// library.cpp : An example of using inheritance and composition

#include "stdafx.h"
#include <iostream>
#include <fstream>
#include <string>
using namespace std;

const int LIBSIZE = 2;  // number of things in the library

// this represents the generic object in the collection
// It is the parent class of the two classes that will actually be stored in the
// container. There will be no instances of this class.
class holding {
public:
	holding() {};
	holding(string n, int l, double c) { title = n; location = l; cost = c;}
	string get_title() { return title;}
	int get_location() { return location;};
	double get_cost() { return cost;}
	void set_title(string s) { title = s;}
	void set_location(int l) { location = l;}
	void set_cost( double c) { cost = c;}
	void print()
	{
		cout << "Title: " << title;
		cout << " Cost: " << cost << " Location:" << location << endl;
	}

protected:
	string title;  // title
	int location;  // shelf number
	double cost;   // cost in dollars
}; // holding

class book : public holding {
public:
	book(string a, int p) {
		author = a; pages=p;
	}
	book(string a, int p, int loc, double c, string t) {
		author = a; pages=p;
		location = loc; cost = c; title = t;
	}
	int get_pages() { return pages;}
	string get_author() { return author;}
	void set_pages(int p) { pages = p;}
	void set_author(string a) { author = a;}
	void print() 
	{
		holding::print();  // run the parent version
		cout << " written by " << author;
		cout << " with " << pages << " pages." << endl;
		cout << "----------------------" << endl;
	} 

protected:
	string author;
	int pages;
}; // book


class video : public holding {
public:
	video(string a, int p) {
		director = a; length=p;
	}
	video(string a, int p, int loc, double c, string t) {
		director = a; length=p;
		location = loc; cost = c; title = t;
	}
	int get_length() { return length;}
	string get_director() { return director;}
	void set_length(int p) { length = p;}
	void set_director(string a) { director = a;}
	void print() 
	{
		holding::print(); // run the parent version
		cout << " Directed by " << director;
		cout << " length " << length << endl;
		cout << "----------------------" << endl;
	}
private:
	string director;
	int length;
}; // video

class library {
public:
	void load_data();
	void print_library();
private:
	holding  L[LIBSIZE];  // container of stuff
}; // library

void
library::load_data() 
{
	ifstream datain;
	char type;
	string author,director,title;
	int pages,location,length;
	double cost;
	int count = 0;
	holding *h;

	// open the data file.
	// There is one line per data entry
	// a book entry starts with a 'b' as the first entry and a video
	// starts with a 'v'.
	// Each then has the three holding data first in this order :
	//  title,location,cost
	//  one per line.
	//  These are then followed by the data that are unique to the 
	//  book and video classes.
	
	// first open the data file and check that it worked
	datain.open("books.txt");
	if(!datain) {
		cerr << "file open failed" << endl;
		exit(1);
	}

	// we loop forever so that we can check the eof status after the
	// first read. This prevents using the last record twice which happens if we use
	// while(!datain.eof()
	while(true) {
		datain >> type;
		if(datain.eof()) break;   // end of file?
		// now read the common data
		getline(datain,title,'$');
		title.erase(0,1);  // delete leading newline
		datain >> location;
		datain >> cost;

		if(type == 'b') {  // book record
			getline(datain,author,'$');
			author.erase(0,1);  // delete leading newline
			datain >> pages;
			h = new book(author,pages,location,cost,title);
		}
		else { // video
			getline(datain,director,'$');
			director.erase(0,1);  // delete leading newline
			datain >> length;
			h = new video(director,length,location,cost,title);
		}
		L[count++] = *h; // add the new object to the container
	} // read data
	datain.close();
} // load_data

// loop over the object in the container and print them.
// Note that the iterator is o holding iterator. Since the print
// function is virtual, we will call the print() method that is appropriate
// for the actual object the iterator points at, not the one that matches
// the type.
void
library::print_library()
{
	cout << "\n" << "List of titles " << endl;
	for(int i =0; i < LIBSIZE; i++) {
		L[i].print();
	}
} // print_library

int 
main(int argc, char* argv[])
{
	library L;
	L.load_data();
	L.print_library();
	return 0;
}
